game_list = {
    "games": [
      "4story",
      "8bitmmo",
      "9dragons",
      "9lives-arena",
      "a-tale-in-the-desert",
      "a3",
      "a3-still-alive",
      "aberoth",
      "ace-online",
      "achaea",
      "ad2460",
      "adventure-land",
      "adventure-quest-3d",
      "adventurequest-worlds",
      "aetolia-the-midnight-age",
      "age-of-conan-unchained",
      "age-of-the-four-clans",
      "age-of-wushu",
      "age-of-wushu-dynasty",
      "agents-of-aggro-city-online",
      "aika",
      "aion",
      "aion-classic",
      "airside-andy",
      "akanbar",
      "albion-online",
      "allods-online",
      "alphadia-genesis",
      "anarchy-online",
      "angels-online",
      "angry-birds-epic",
      "animal-crossing-new-horizons",
      "anime-ninja",
      "anime-pirates",
      "anocris",
      "anthem",
      "antilia",
      "apb-reloaded",
      "apex-legends",
      "aranock-online",
      "arcane-legends",
      "arcane-waters",
      "arcfall",
      "archeage",
      "archeage-begins",
      "archeage-unchained",
      "archeblade",
      "ark-survival-evolved",
      "armed-heroes-2",
      "armored-warfare",
      "artifact",
      "ashen-empires",
      "ashes-of-creation-apocalypse",
      "assassins-creed-valhalla",
      "astaria",
      "asteidus",
      "astellia",
      "astonia-reborn",
      "astral-terra",
      "astro-empires",
      "astro-lords-oort-cloud",
      "atlantica-online",
      "atlas",
      "atlas-reactor",
      "atlas-rogues",
      "atom-rpg",
      "aura-kingdom",
      "avalon-the-legend-lives",
      "avatar-star",
      "avernum-2-crystal-souls",
      "avowed",
      "azulgar",
      "baldurs-gate-iii",
      "barons-of-the-galaxy",
      "battle-chasers",
      "battle-dawn",
      "battle-dawn-galaxies",
      "battleborn",
      "battlerite",
      "black-aftermath",
      "black-desert-mobile",
      "black-desert-online",
      "black-legend",
      "black-prophecy-tactics-nexus-conflict",
      "blackfaun",
      "blackguards",
      "blackguards-2",
      "blacklight-retribution",
      "blade-and-soul",
      "blade-and-soul-2",
      "blade-and-soul-revolution",
      "blankos-block-party",
      "bleach-online",
      "bless-mobile",
      "bless-unleashed",
      "block-story",
      "bloodborne",
      "bloodline-champions",
      "blossom-and-decay",
      "bombshell",
      "book-of-demons",
      "book-of-travels",
      "boot-hill-heroes",
      "borderlands-2",
      "borderlands-3",
      "borderlands-the-pre-sequel",
      "bound-by-flame",
      "boundless",
      "bounty-bay-online",
      "bounty-hounds-online",
      "bravada",
      "brave-nine",
      "bravely-default",
      "bravely-default-ii",
      "bravely-second",
      "cabal-online",
      "call-of-duty-warzone",
      "call-of-gods",
      "camelot-unchained",
      "caravan-stories",
      "cardlife",
      "casinorpg",
      "castlot",
      "celtic-heroes",
      "champions-of-regnum",
      "champions-of-titan",
      "champions-online",
      "child-of-light",
      "chroma-squad",
      "chronicles-of-denzar",
      "chrono-odyssey",
      "chrono-tales",
      "chrono-wars",
      "citizens-of-earth",
      "city-of-titans",
      "clan-lord",
      "clash-of-avatars",
      "clash-of-clans",
      "closers",
      "colonies-online",
      "command-and-conquer-tiberium-alliances",
      "conan-exiles",
      "conquer-online",
      "conquerors-blade",
      "continent-of-the-ninth-seal",
      "core-exiles",
      "cosmicbreak-universal",
      "costume-quest-2",
      "crimson-desert",
      "cronous",
      "crossfire",
      "crossout",
      "crowfall",
      "crucible",
      "crusader-kings-iii",
      "crystal-saga",
      "crystal-saga-2",
      "cubic-castles",
      "cuisine-royale",
      "cyberpunk-2077",
      "daimonin",
      "damascus",
      "dark-age-of-camelot",
      "dark-ages",
      "dark-and-light",
      "dark-genesis",
      "dark-knight",
      "dark-legends",
      "dark-souls-3",
      "dark-souls-ii",
      "darkest-dungeon",
      "darkorbit-reloaded",
      "darkspace",
      "darkstory-online",
      "darkwind-war-on-wheels",
      "dauntless",
      "dayz",
      "dc-legends",
      "dc-universe-online",
      "ddtank",
      "dead-by-daylight",
      "dead-frontier",
      "dead-frontier-ii",
      "dead-island",
      "dead-island-riptide",
      "dead-maze",
      "dead-state",
      "deadside",
      "deepworld",
      "defend-the-night",
      "defiance",
      "defiance-2050",
      "dekaron",
      "deorum-online",
      "desert-operations",
      "destiny",
      "destiny-2",
      "destiny-of-ancient-kingdoms",
      "destinys-sword",
      "deus-ex-mankind-divided",
      "diablo-3",
      "diablo-ii-resurrected",
      "diablo-immortal",
      "diablo-iv",
      "digimon-masters-online",
      "dino-storm",
      "disintegration",
      "divergence-online",
      "divinity-original-sin-2",
      "divinity-original-sin",
      "dk-online",
      "dofus",
      "dokev",
      "doom-eternal",
      "dota-2",
      "drachenblut2",
      "dragon-age-4",
      "dragon-age-inquisition",
      "dragon-awaken",
      "dragon-blood",
      "dragon-eternity",
      "dragon-fin-soup",
      "dragon-lord",
      "dragon-nest",
      "dragon-of-legends",
      "dragon-pals",
      "dragon-raja-online",
      "dragon-saga",
      "dragons-dogma-dark-arisen",
      "dragonfable",
      "dragons-and-titans",
      "drakengard-3",
      "drakensang-online",
      "dransik-classic",
      "dreadnought",
      "dream-of-mirror-online",
      "dreamworld",
      "drifters-loot-the-galaxy",
      "duelyst",
      "dungeon-fighter-online",
      "dungeon-hero",
      "dungeon-hunter-5",
      "dungeon-of-the-endless",
      "dungeon-party",
      "dungeons-and-dragons-online",
      "dungeons-and-dragons-dark-alliance",
      "dv8-exile",
      "dying-light",
      "dynastica",
      "dynasty-of-the-magi",
      "earthlock-festival-of-magic",
      "echo-of-soul",
      "echo-of-soul-phoenix",
      "eden-eternal",
      "eden-falling",
      "edenbrawl",
      "edengrad",
      "edge-of-space",
      "elden-ring",
      "eldevin",
      "elite-dangerous",
      "elsword",
      "elvenar",
      "elyon",
      "elysian-war",
      "ember-sword",
      "empire",
      "empire-four-kingdoms",
      "empire-of-sports",
      "empire-universe-3",
      "empire-warzone",
      "empire-revenant",
      "enlisted",
      "entropia-universe",
      "epicduel",
      "erectus-the-game",
      "eredan",
      "erepublik",
      "escape-from-tarkov",
      "estogan",
      "eternal-fury",
      "eternal-lands",
      "eternal-magic",
      "etrian-mystery-dungeon",
      "etrian-odyssey-2-untold-the-fafnir-knight",
      "eudemons-online",
      "eve-online",
      "eve-valkyrie",
      "everember-online",
      "evernight",
      "everquest",
      "everquest-ii",
      "everspace-2",
      "evidyon-no-mans-land",
      "evony",
      "excalibur",
      "exile-online",
      "fire-special-ops",
      "factions-origins-of-malu",
      "fallen-earth",
      "fallen-sword",
      "fallout-4",
      "fallout-76",
      "fantasy-life",
      "fantasy-realm-online-moon-haven",
      "fantasy-tales-online",
      "fantasy-worlds-rhynn",
      "fasaria-world-online",
      "fear-the-night",
      "fear-the-wolves",
      "fearless-fantasy",
      "fellow-eternal-clash",
      "felspire",
      "fiesta-online",
      "final-fantasy-type-0-hd",
      "final-fantasy-vii-remake",
      "final-fantasy-xx-2-remaster",
      "final-fantasy-xi",
      "final-fantasy-xv",
      "final-fantasy-xvi",
      "five-guardians-of-david",
      "flamefrost-legacy",
      "florensia",
      "flyff-gold",
      "football-superstars",
      "force-of-arms",
      "forge-of-empires",
      "forsaken-world",
      "fortnite",
      "foxhole",
      "fragmented",
      "freeworld-apocalypse-portal",
      "furcadia",
      "galaxy-online-2",
      "galaxy-warfare",
      "game-of-thrones-winter-is-coming",
      "games-of-glory",
      "gates-of-andaron",
      "gauntlet",
      "gekkeiju-online",
      "generals-art-of-war",
      "genshin-impact",
      "ghost-of-tsushima",
      "gladiatus",
      "gloria-victis",
      "glory-of-gods",
      "goal-line-blitz",
      "godfall",
      "godswar-online",
      "graal-kingdoms",
      "gran-saga",
      "granado-espada-online",
      "grand-fantasia",
      "grand-theft-auto-online",
      "grav",
      "grepolis",
      "grim-dawn",
      "grounded",
      "guardians-of-divinity",
      "guardians-of-ember",
      "guild-wars",
      "guild-wars-factions",
      "guild-wars-nightfall",
      "habbo",
      "halosphere2",
      "hand-of-fate",
      "haven-and-hearth",
      "hawken",
      "heart-forth-alicia",
      "hearthstone",
      "helbreath",
      "hellgate",
      "hello-kitty-online",
      "hero-online",
      "hero-wars",
      "hero-zero",
      "heroes-and-generals",
      "heroes-and-legends-conquerors-of-kolhar",
      "heroes-evolved",
      "heroes-in-the-sky",
      "heroes-of-atlan",
      "heroes-of-gaia",
      "heroes-of-newerth",
      "heroes-of-the-storm",
      "herosmash",
      "hex-shards-of-fate",
      "highlands",
      "hob",
      "hogwarts-legacy",
      "hood-outlaws-and-legends",
      "hordes",
      "horizon-zero-dawn",
      "hostile-space",
      "humankind",
      "hunters-arena-legends",
      "hustle-castle",
      "hyper-universe",
      "i-am-setsuna",
      "icewind-dale-enhanced-edition",
      "ikariam",
      "illyriad",
      "ilysia",
      "immortal-day",
      "immortals-fenyx-rising",
      "imperian",
      "infantry-online",
      "inferna",
      "infestation-survivor-stories",
      "infinite-fleet",
      "inked-mafia",
      "interstellar-war",
      "ironsight",
      "island-forge",
      "islandoom",
      "istaria-chronicles-of-the-gifted",
      "kal-online",
      "key-to-heaven",
      "kingdom-come-deliverance",
      "kingdom-of-drakkar",
      "kingdom-under-fire-ii",
      "kingdom-wars",
      "kingdoms-of-amalur-re-reckoning",
      "kingory",
      "kings-and-heroes",
      "kings-and-legends",
      "kings-era",
      "kings-of-the-realm",
      "kingshunt",
      "kingsroad",
      "knight-online",
      "kurtzpel",
      "kyn",
      "la-tale",
      "last-chaos",
      "last-epoch",
      "last-oasis",
      "league-of-angels",
      "league-of-angels-heavens-fury",
      "league-of-angels-ii",
      "league-of-angels-iii",
      "league-of-legends",
      "league-of-legends-wild-rift",
      "leap-of-fate",
      "left-to-survive",
      "legend-of-grimrock-2",
      "legend-of-zelda-breath-of-the-wild",
      "legends-of-aria",
      "legends-of-equestria",
      "legends-of-honor",
      "legends-of-persia",
      "legends-of-runeterra",
      "liberators",
      "lichdom-battlemage",
      "life-beyond",
      "life-is-feudal",
      "life-of-rome",
      "light-of-nova",
      "line-of-defense",
      "lineage-2",
      "lineage-2-aden",
      "lineage-2-essence",
      "lineage-2-revolution",
      "lineage-eternal-twilight-resistance",
      "livelock",
      "lord-of-chains",
      "lord-of-the-rings-online",
      "lords-of-the-fallen",
      "lords-of-the-fallen-2",
      "lost-dimension",
      "lucent-heart",
      "luckcatchers",
      "luminary-rise-of-the-goonzu",
      "lusternia-age-of-ascension",
      "mabinogi",
      "mad-max",
      "mad-world-mmo",
      "maelstrom",
      "mafiashot",
      "magic-duels-origins",
      "magic-legends",
      "magic-the-gathering-arena",
      "manarocks",
      "maplestory",
      "maplestory-2",
      "maplestory-m",
      "march-to-rome",
      "margonem",
      "marvel-future-revolution",
      "marvels-avengers",
      "marvel-puzzle-quest",
      "mass-effect-legendary-edition",
      "mass-effect-andromeda",
      "mechwarrior-online",
      "meridian-59",
      "merlin",
      "metal-war-online",
      "metin-2",
      "middle-earth-shadow-of-mordor",
      "might-and-magic-heroes-online",
      "might-and-magic-x-legacy",
      "milmo",
      "minecraft",
      "minecraft-dungeons",
      "ministry-of-war",
      "mirage-online-classic",
      "monster-hunter-4-ultimate",
      "monster-hunter-rise",
      "monster-hunter-world",
      "monstermmorpg",
      "mordhau",
      "mortal-online",
      "mortal-shell",
      "mount-and-blade-ii-bannerlord",
      "mu-legend",
      "mu-online",
      "mu-origin",
      "mu-origin-2",
      "my-lands",
      "myst-online-uru-live",
      "myth-war-2",
      "mythborne",
      "mytheon",
      "naruto-online",
      "naval-action",
      "navy-field",
      "navy-field-2",
      "necropolis",
      "nemexia",
      "neocron-2",
      "nether",
      "neverwinter",
      "new-dawn",
      "new-world",
      "next-island",
      "nexus-the-kingdom-of-the-winds",
      "nioh-2",
      "no-mans-sky",
      "nords-heroes-of-the-north",
      "nostale",
      "nova-genesis",
      "oberin",
      "odin-quest",
      "ogre-island",
      "old-world",
      "omega-zodiac",
      "omerta-3",
      "one-piece-online",
      "one-piece-online-2-pirate-king",
      "online-boxing-manager",
      "online-tennis-manager",
      "orake",
      "orbusvr",
      "orcs-must-die-3",
      "order-and-chaos-2-redemption",
      "order-and-chaos-online",
      "order-of-magic",
      "origins-return",
      "orions-belt",
      "osiris-new-dawn",
      "otherland",
      "out-of-reach",
      "outlaws-of-the-old-west",
      "outriders",
      "outwar",
      "overkings",
      "oversoul",
      "overwatch",
      "oz-broken-kingdom",
      "pagan-online",
      "paladins-champions-of-the-realm",
      "palia",
      "panzar",
      "past-fate",
      "path-of-exile",
      "pathfinder-online",
      "perfect-world-international",
      "perfect-world-mobile",
      "persona-5",
      "persona-5-royal",
      "persona-q-shadow-of-the-labyrinth",
      "phantasy-star-online-2",
      "phoenix-dynasty-2",
      "piercing-blow",
      "pillars-of-eternity",
      "pillars-of-eternity-2-deadfire",
      "pirate-arena",
      "pirate-galaxy",
      "pirate-king-online",
      "pirate-storm",
      "pirate101",
      "pirates-of-the-burning-sea",
      "pirates-tides-of-fortune",
      "planeshift",
      "planet-arkadia",
      "planet-calypso",
      "planetside-2",
      "playable-worlds",
      "playerunknowns-battlegrounds",
      "pocket-legends",
      "pokemon-go",
      "pokemon-xy",
      "population-zero",
      "portal-knights",
      "pox-nora",
      "predator-hunting-grounds",
      "preta-vendetta-rising",
      "prime-world",
      "priston-tale",
      "profane",
      "project-genom",
      "project-gorgon",
      "project-ion",
      "project-x-zone-2",
      "project-zomboid",
      "prosperous-universe",
      "quest-for-infamy",
      "r2-online-reign-of-revolution",
      "radical-heights",
      "rage-of-3-kingdoms",
      "ragewar",
      "ragnarok-online",
      "ragnarok-online-ii",
      "ragnarok-online-transcendence",
      "ragnarok-origin",
      "raid-shadow-legends",
      "rail-nation",
      "rakion",
      "ran-online",
      "rangers-of-oblivion",
      "rappelz",
      "rappelzsea",
      "rapture-rejects",
      "realm-of-the-mad-god",
      "realm-royale",
      "rebel-galaxy",
      "rebirth-fantasy",
      "record-of-lodoss-war-online",
      "red-dead-online",
      "red-stone",
      "redemptions-guild",
      "redfall",
      "regnum-online",
      "remnant-from-the-ashes",
      "rend",
      "requiem-memento-mori",
      "revelation-online",
      "riders-of-icarus",
      "riders-republic",
      "rift",
      "rimworld",
      "ring-of-elysium",
      "riotzone",
      "rise",
      "rise-of-the-tycoon",
      "risen-3-titan-lords",
      "rivality",
      "roblox",
      "robocraft",
      "rocket-league",
      "rogalia",
      "rogue-company",
      "rohan-blood-feud",
      "romadoria",
      "rosh-online",
      "roto-x",
      "royal-quest",
      "rulers-of-the-sea",
      "rumble-fighter",
      "runes-of-magic",
      "runescape",
      "rust",
      "ryzom",
      "s4-league",
      "sacred-3",
      "saga",
      "salem",
      "samutale",
      "savage-lands",
      "scarlet-nexus",
      "scars-of-honor",
      "scavengers",
      "scions-of-fate-yulgang",
      "sea-of-thieves",
      "seal-online-blades-of-destiny",
      "seas-of-gold",
      "second-life",
      "serenia-fantasy",
      "seven-seas-saga",
      "shadow-arena",
      "shadowbound",
      "shadowgate",
      "shadowgun-legends",
      "shadowrun-chronicles",
      "shadowrun-returns",
      "shadowrun-hong-kong",
      "shadowverse",
      "shaiya",
      "shattered-galaxy",
      "shin-megami-tensei-devil-survivor-2-record-breaker",
      "ship-of-heroes",
      "shot-online",
      "shroud-of-the-avatar",
      "siegefall",
      "siegelord",
      "silkroad-online",
      "skull-and-bones",
      "skyclimbers",
      "skyforge",
      "skylanders-battlecast",
      "smite",
      "solasta-crown-of-the-magister",
      "soldiers-inc",
      "soul-order-online-2",
      "soulworker",
      "south-park-stick-of-truth",
      "space-wars-interstellar-empires",
      "sparta-war-of-empires",
      "spellbreak",
      "spellgear",
      "spellstone",
      "sphere-3",
      "spiral-knights",
      "splitgate-arena-warfare",
      "star-citizen",
      "star-colony",
      "star-conflict",
      "star-crusade-war-for-the-expanse",
      "star-legends",
      "star-ocean-5-integrity-and-faithlessness",
      "star-sonata-2",
      "star-stable",
      "star-trek-online",
      "star-wars-battlefront-ii",
      "star-wars-squadrons",
      "star-wars-the-old-republic",
      "starbase",
      "starborne",
      "starborne-frontiers",
      "starbound",
      "starfield",
      "starport-galactic-empires",
      "stash-no-loot-left-behind",
      "state-of-decay",
      "steambirds-alliance",
      "steamcraft",
      "steinworld",
      "stoneage-world",
      "storm-riders",
      "stormfall-age-of-war",
      "stormthrone",
      "stranger-of-paradise",
      "stronghold-kingdoms",
      "styx-master-of-shadows",
      "supernova",
      "supremacy-1914",
      "supreme-destiny",
      "survarium",
      "sword-coast-legends",
      "swords-of-legends-online",
      "tales-of-symphonia-chronicles",
      "tales-of-zestiria",
      "talisman-online",
      "tamer-saga",
      "tantra-online",
      "tantra-rumble",
      "temtem",
      "tentlan",
      "tera",
      "terraria",
      "thang-online",
      "the-4th-coming",
      "the-aetherlight-chronicles-of-the-resistance",
      "the-ascent",
      "the-banner-saga",
      "the-banner-saga-2",
      "the-black-death",
      "the-black-watchmen",
      "the-crew",
      "the-crew-2",
      "the-division",
      "the-division-2",
      "the-dwarves",
      "the-elder-scrolls-blades",
      "the-end",
      "the-epic-might",
      "the-exiled",
      "the-hammers-end",
      "the-incredible-adventures-of-van-helsing",
      "the-incredible-adventures-of-van-helsing-2",
      "the-incredible-adventures-of-van-helsing-iii",
      "the-legend-of-legacy",
      "the-legend-of-pirates-online",
      "the-mighty-quest-for-epic-loot",
      "the-mob-wars",
      "the-outer-worlds",
      "the-pride-of-taern",
      "the-realm-online",
      "the-repopulation",
      "the-settlers-online",
      "the-skies",
      "the-technomancer",
      "the-wagadu-chronicles",
      "the-waylanders",
      "the-west",
      "the-witcher-3-wild-hunt",
      "there",
      "therian-saga",
      "thunder-run-firestrike",
      "tibia",
      "tibia-micro-edition",
      "tiger-knight",
      "titan-siege",
      "titanreach",
      "titans-of-time",
      "torchlight-2",
      "torchlight-iii",
      "torment-tides-of-numenera",
      "total-domination",
      "total-war-saga-troy",
      "total-war-three-kingdoms",
      "total-war-arena",
      "transistor",
      "trapped-dead-lockdown",
      "travian",
      "travian-kingdoms",
      "tree-of-life",
      "tree-of-savior",
      "trials-of-mana",
      "tribal-wars",
      "tribes-of-midgard",
      "tribes-ascend",
      "trove",
      "tug",
      "turf-battles",
      "twin-saga",
      "tynon",
      "tyranny",
      "ufo-online",
      "ultima-online",
      "uncharted-waters-online",
      "underlight-clash-of-dreams",
      "underworld-ascendant",
      "unification-wars",
      "unlimited-ninja",
      "utopia",
      "v-rising",
      "v4",
      "vainglory",
      "valdis-story-abyssal-city",
      "valheim",
      "valiance-online",
      "valnir-rok",
      "valorant",
      "vampire-the-masquerade-bloodlines-2",
      "vampyr",
      "vendetta-online",
      "victor-vran",
      "victory-age-of-racing",
      "vikings-war-of-clans",
      "villagers-and-heroes",
      "vindictus",
      "virtonomics-economics-game-online",
      "visions-of-zosimos",
      "voidexpanse",
      "voyage-century-online",
      "wakfu",
      "war-of-mercenaries",
      "war-thunder",
      "war2-glory",
      "warflare",
      "wargame1942",
      "warhammer-40k-inquisitor-martyr",
      "warhammer-chaosbane",
      "warhammer-odyssey",
      "warmonger",
      "warp-nexus",
      "wartune",
      "wasteland-2",
      "wasteland-3",
      "waven",
      "we-ride",
      "weapons-of-mythology",
      "wild-terra",
      "wild-west-online",
      "wind-of-luck-arena",
      "wings-of-destiny",
      "winning-putt",
      "with-your-destiny",
      "wizard101",
      "wolcen-lords-of-mayhem",
      "world-golf-tour",
      "world-of-kung-fu",
      "world-of-tanks",
      "world-of-trinketz",
      "world-of-warcraft",
      "world-of-warplanes",
      "world-of-warriors",
      "world-of-warships",
      "world-war-online",
      "worldalpha",
      "worlds-adrift",
      "wurm-online",
      "wwii-online",
      "xcom-chimera-squad",
      "xenoblade-chronicles-3d",
      "xenoblade-chronicles-definitive-edition",
      "xenoblade-chronicles-x",
      "xsyon-prelude",
      "xulu",
      "yohoho-puzzle-pirates",
      "z1-battle-royale",
      "zeal",
      "zenith",
      "zodiac"
    ]
  }


from enum import Enum


class StatusCode(Enum):
    Success = 200
    PartialContent = 206
    NotFound = 404


# BASE URL of mmorpg.com
BASE_URL = "https://www.mmorpg.com/"

# Distribution results from site.
distributions = ["Download", "Browser", "Retail"]

# Business model results from site.
business_models = ["Buy to Play", "Free", "Hybrid", "Item Mall", "Subscription"]

# Platform results from site.
platforms = ["3DS", "Android", "DreamCast", "Google Stadia", "iOS", "Linux", 
                "Mac", "Mobile Devices", "PC", "Playstation 2", "Playstation 4",
                "Playstation 5", "PS3", "Switch", "Vita", "Wii", "Wii U", "XBox",
                "XBox 360", "Xbox One", "Xbox Series X/S"]



import json
from typing import Optional


class GameDetail:
    """ Creates game_detail object. """
    def __init__(self,
                 response: Optional[int] = None,
                 response_detail: Optional[str] = None,
                 name: Optional[str] = None,
                 publisher: Optional[str] = None,
                 official_site: Optional[str] = None,
                 genre: Optional[str] = None,
                 setting: Optional[str] = None,
                 pvp: Optional[str] = None,
                 classification: Optional[str] = None,
                 engine: Optional[str] = None,
                 style: Optional[str] = None,
                 developer: Optional[str] = None,
                 status: Optional[str] = None,
                 release_date: Optional[str] = None,
                 distribution: Optional[list] = [],
                 business_model: Optional[list] = [],
                 platforms: Optional[list] = [],
                 rating: Optional[float] = None,
                 vote_count: Optional[int] = None,
                 overview: Optional[str] = ""):
        self.result = {"status" : response, "detail" : response_detail}
        self.name = name
        self.publisher = publisher
        self.official_site = official_site
        self.genre = genre
        self.setting = setting
        self.pvp = pvp
        self.classification = classification
        self.engine = engine
        self.style = style
        self.developer = developer
        self.status = status
        self.release_date = release_date
        self.distribution = distribution
        self.business_model = business_model
        self.platforms = platforms
        self.rating = {"rating": rating, "vote_count": vote_count}
        self.overview = overview

    def to_json(self) -> str:
        """Returns game_detail object as Json object.

        :return: a dictionary of game_details
        """
        return json.dumps(self, default=lambda o: o.__dict__,
                          indent=4)



from bs4 import BeautifulSoup
from mmorpg_scraper.scraper.game_detail import GameDetail
from mmorpg_scraper.scraper.request import SendRequest
from mmorpg_scraper.scraper.exceptions import ParsingError
from mmorpg_scraper.scraper.constants import (
    StatusCode,
    distributions,
    business_models,
    platforms
)


class Scraper:
    def __init__(self, game_name: str):
        """
        Initializes GameDetail object.
        Sends request with given game_name.
        Parses response.content with bs4.
           
        :param game_name: String game_name to 
        execute process.
        """
        self.game_detail = GameDetail()
        self.request = SendRequest(game_name)
        self.soup = BeautifulSoup(self.request.response.content, 'html.parser')


    def fetch_game_details(self) -> str: # TODO: Improvements needed. Exceptions should handled.
        """
        Executes parsing to mmorpg.com.
        :return: game detail object as JSON.
        """
        try:
            self.parse_name()
            self.parse_publisher_and_official_site()
            self.parse_genre()
            self.parse_distribution_and_business_model()
            self.parse_platform()
            self.parse_rating()
            self.parse_vote_count()
            self.parse_overview()
        except:
            raise ParsingError("Can not parse publisher.")
        finally:
            self.set_status_code()
            return self.game_detail.to_json()


    def parse_name(self) -> None:
        """
        Parse name for game_detail.name
        """
        try:
            self.game_detail.name  = self.soup.find("div", {"class": "game-info__top-title-wrap"}).h1.text
        except:
            pass


    def parse_publisher_and_official_site(self) -> None:
        """
        Parse publisher and official site for
        game_detail.publisher and game_detail.official_site
        """
        try:
            self.game_detail.publisher = self.soup.find("p", {"title": "Publisher"}).text.partition("|")[0]
            self.game_detail.official_site = self.soup.find("p", {"title": "Publisher"}).a['href']
        except:
            pass     


    def parse_genre(self) -> None:  # TODO: IMPROVEMENTS NEEDED.
        """
        Parse genre, setting, pvp, classification, engine, style,
        developer, status and release date for game_detail object.
        """
        try:
            strongs = [strong.text for strong in self.soup.find(class_='game-info__middle').find_all('strong')]
            self.game_detail.genre = strongs[0]
            self.game_detail.setting = strongs[1]
            self.game_detail.pvp = strongs[2]
            self.game_detail.classification = strongs[3]
            self.game_detail.engine = strongs[4]
            self.game_detail.style = strongs[5]
            self.game_detail.developer = strongs[6]
            self.game_detail.status = strongs[7]
            self.game_detail.release_date = strongs[8]
        except:
            pass

    def parse_distribution_and_business_model(self) -> None:
        """
        Parse distribution and business model for game_detail object.
        """
        try:
            for element in self.soup.find(class_='game-info__middle').find_all('i'):
                if element.get('title') in distributions and element.get('title') not in self.game_detail.distribution:
                    self.game_detail.distribution.append(element.get('title'))
                elif element.get('title') in business_models and element.get('title') not in self.game_detail.business_model:
                    self.game_detail.business_model.append(element.get('title'))
        except:
            pass

    def parse_platform(self) -> None:
        """
        Parse platform for game_detail.platform
        """
        try:
            for element in self.soup.find(class_='game-info__button').find_all('i'):   
                if element.get('title') in platforms and element.get('title') not in self.game_detail.platforms:
                    self.game_detail.platforms.append(element.get('title'))
        except:
            pass
        

    def parse_rating(self) -> None:
        """
        Parse rating for game_detail.rating["rating"]
        """
        try:
            self.game_detail.rating["rating"] = float(self.soup.find(class_='game-info__button').find('strong').text)
        except:
            pass


    def parse_vote_count(self) -> None:
        """
        Parse vote_count for game_detail.rating["vote_count"]
        """
        try:
            self.game_detail.rating["vote_count"] = int(self.soup.find(class_='user-rating__head-title').find('span').text)
        except:
            pass


    def parse_overview(self) -> None:
        """
        Parse overview for game_detail.overview
        """
        try:
            for element in self.soup.find('article'):
                self.game_detail.overview += element.text
        except:
            pass

    def set_status_code(self) -> None: # TODO: When expection raises, add can not parsed category to detail.
        """
        Parse status code and details for game_detail.result.
        """
        if self.game_detail.result["detail"]:
            self.game_detail.result["status"] = StatusCode.PartialContent.value
        else:
            self.game_detail.result["status"], self.game_detail.result["detail"] = StatusCode.Success.value, "Success"



import sys
sys.path.append('.')
from mmorpg_scraper.app import  fetch_game_list, fetch_game_details
from datetime import datetime

# TODO: Create more test.

start = datetime.now()
"""
Return game_details

result = fetch_game_details('4story')
print(result)
"""


"""
Return game_list

result = fetch_game_list()
print(result)

"""


# with open("game_result2.txt", "r") as game_res:
#     lines = game_res.readlines()
#     lines = [line.rstrip() for line in lines]

# with open("test_json_result.txt", "w+") as json_result:
#     for i in lines:
#         i = i.strip("\'")
#         res = app(i)
#         json_result.write(res)
#         json_result.write("\n\n")
#         print(i + "done")
# a = app('elder-scrolls-online')
# print(a)  

end = datetime.now()
print(end-start)



import random
import string

def random_char(char_num: int) -> str:
    """
    Random char generator to use with email.

    :param char_num: character number as integer to generate random letters.
    :return: Random string with given char_num length.
    """
    return ''.join(random.choice(string.ascii_letters) for _ in range(char_num))

def generate_mail() -> str:
    """
    Generates and returns random email for headers.

    :return: Returns generated random email as string.
    """
    return random_char(random.randint(5,8))+"@"+random_char(random.randint(3,6))+".com"

def headers() -> dict:
    """
    Generates headers to use in request.

    :return: headers as dict.
    """
    headers = { 
        'User-Agent': 'My User Agent 1.0', 
        'From': generate_mail()
    }
    return headers



import requests
from mmorpg_scraper.scraper.headers_generator import headers
from mmorpg_scraper.scraper.constants import BASE_URL
from mmorpg_scraper.scraper.exceptions import ExtraHTTPError, NotFoundError


class SendRequest:
    def __init__(self, game_name: str):
        """ 
        Sends a get request with given game_name.
        Returns 404 or Status code if fails.
        Initializes response.
        
        :param game_name: game name to request.
        """
        try:
            self.response = requests.get(BASE_URL+game_name, headers=headers())
            if self.response.status_code == 404:
                raise NotFoundError("Url not found(404).")
        except:
            raise ExtraHTTPError(
                "Game not found. Status code {} returned.".format(self.response.status_code)
            )



from typing import Dict, Optional
from mmorpg_scraper.scraper.scraper import Scraper
from mmorpg_scraper.scraper.game_list import game_list


def fetch_game_details(game_name: Optional[str] = None) -> str:
    """
    Fetches game details from mmorpg.com 

    :param game_name: String game name e.g '4story'
    :return: game_detail json object as string
    """
    scraper = Scraper(game_name)
    return scraper.fetch_game_details()

def fetch_game_list() -> dict:
    """
    Fetches game_list.

    :return: game_list as dict. game_list["games"] -> list
    """
    return game_list
